// top

/*
-------------------------------------------------------------------------------

This file is part of the hardware description for the Propeller 1 Design.

The Propeller 1 Design is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your option)
any later version.

The Propeller 1 Design is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
the Propeller 1 Design.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------
*/

//RJA Restarting with the Pipistrello version after reading forum and merging in SaucySoliton parts
//RJA merging SaucySoliton's top.v, tim.v, and features with source for A( version P123 with PortB 
//RJA adding portb stuff from the A9 version top.tdf
//RJA we don't seem to be able to preload cog ram, not sure why not


`include "dig.v"


module              top
(
input               CLK, //RJA  100 MHz
output       [3:0]  LED,

input           RST_BTN,
input           uart_txd_in,
output          UART_TXD,

//With PortB (comment out this section if not using PortB)
output                led0_r,  //PortB pin [10]
output                led0_g,  //PortB pin [11]
output                led0_b,  //PortB pin [12]
output                led1_r,  //PortB pin [13]
output                led1_g,  //PortB pin [14]
output                led1_b,  //PortB pin [15]
input                 [3:0] SW,  //PortB pins [23:20]
input                 [3:0] BTN, //PortB pins [19:16]
inout                 [7:0] jb, //PortB pins [31:24]
inout 	              [41:0] ck_io  //top ten on Port B [9:0], lower 32 on Port A

//Without PortB  (comment out this section if using PortB)
//inout 	              [31:0] ck_io  //top ten on Port B [9:0], lower 32 on Port A


);

`include "features.v"

wire           ftdi_ndtr = RST_BTN;
wire                   ftdi_txd = uart_txd_in;  //FPGA receives from ftdi
wire                  ftdi_rxd =UART_TXD;  //FPGA transmits to ftdi

//
// reg and wire declarations
//
reg                 nres;
wire         [7:0]  cfg;
wire        [31:0]  pin_out, pin_dir;

wire                clkfb, clock_160, clk;
reg         [23:0]  reset_cnt;
reg                 reset_to;
wire                clk_pll;
wire                clk_cog;

wire         [31:0] pin_in;// = io;
`ifdef USE_PORTB
wire         [31:0] pin_inb;// = iob;  //PortB
wire        [31:0]  pin_outb, pin_dirb;  //PortB
`endif

wire [7:0] cogleds;
assign LED[3:0]= cogleds[3:0];


`ifdef USE_PROPPLUG
assign pin_in = ck_io[31:0]; //RJA for Prop Plug
wire inp_resn = ck_io[41];  //Guess we'd use the last pin here for Prop Plug RESn?
`else
assign pin_in = {ftdi_txd, ftdi_rxd, ck_io[29:0]};
wire inp_resn = ftdi_ndtr;
`endif

wire [31:0] io;
assign ck_io[31:0] = io[31:0];

`ifdef USE_PORTB
wire [31:0] iob;
assign iob = {jb[7:0], SW[3:0], BTN[3:0], led1_b, led1_g, led1_r, led0_b, led0_g, led0_r,ck_io[41:32]};  
assign pin_inb = iob;
`endif //USE_PORTB


//
// Clock generation
//

wire [4:0] _dummy_clk;



assign clk=clock_160;//RJA
wire clk_rcfast;
//PLL1 PLL1_inst(clk_25mhz, clock_160, clock_rcfast, clock_80);
 clk_wiz_0 clk_wiz_0_inst
 (// Clock in ports
  // Clock out ports
  .clk_160(clock_160),
  .clk_in1(CLK)
 );

//    
// Clock control (replaces including tim.v)
// 

reg [12:0]  divide;
reg [7:0]   cfgx;
assign clk_pll = ((cfgx[6:5] == 2'b11) && (cfgx[2:0] == 3'b111)) ? clock_160 : divide[11];
assign clk_cog = divide[12];

always @ (posedge clock_160)
    cfgx <= cfg;
    
wire res = ~inp_resn;

always @ (posedge clock_160)
    divide <= divide + 
        {   (cfgx[6:5] == 2'b11 && cfgx[2:0] == 3'b111) || res,
            cfgx[6:5] == 2'b11 && cfgx[2:0] == 3'b110 && !res,
            cfgx[6:5] == 2'b11 && cfgx[2:0] == 3'b101 && !res,
            ((cfgx[6:5] == 2'b11 && cfgx[2:0] == 3'b100) || cfgx[2:0] == 3'b000) && !res,
            ((cfgx[6:5] == 2'b11 && cfgx[2:0] == 3'b011) || (cfgx[5] == 1'b1 && cfgx[2:0] == 3'b010)) && !res,
            7'b0,
            cfgx[2:0] == 3'b001 && !res
        };



//
// Propeller 1 core module
//

dig core (  .nres       (nres),
            .cfg        (cfg),
            .clk_cog    (clk_cog),
            .clk_pll    (clk_pll),
            .pin_in     (pin_in),
            .pin_out    (pin_out),
            .pin_dir    (pin_dir),
`ifdef USE_PORTB			
			.pin_inb     (pin_inb),  //PortB
            .pin_outb    (pin_outb),  //PortB
            .pin_dirb    (pin_dirb),  //PortB
`endif			
			//.px0(px0), //RJA bringing out program counter for troubleshooting
            .cog_led    (cogleds) );

always @ (posedge clk_cog)
    nres <= inp_resn & !cfg[7];

//
// Bidir I/O buffers
//

genvar i;
generate
    for (i=0; i<32; i=i+1)
    begin : iogen
		if (i==30)
		begin	
		  assign io[i] = pin_dir[i] ? pin_out[i] : 1'bZ;
		  assign ftdi_rxd = pin_dir[i] ? pin_out[i] : 1'bZ;
		end
		else
          assign io[i] = pin_dir[i] ? pin_out[i] : 1'bZ;
`ifdef USE_PORTB
        if ((i<16) || (i>23)) //exclude input only		  
		  assign iob[i] = pin_dirb[i] ? pin_outb[i] : 1'bZ;  //PortB
`endif		
    end
endgenerate

endmodule
